import json
import os
import subprocess
import numpy as np

from django.core.management.base import BaseCommand, CommandError

from ...models import SoundFile


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--force',
            action='store_true',
            dest='force',
            default=False,
            help='Create files even if they exist')
        parser.add_argument('--emission',
            dest='emission',
            metavar='EMISSION',
            default=None,
            help='Process files belonging to emission only')
        parser.add_argument('--episode',
            dest='episode',
            metavar='EPISODE',
            default=None,
            help='Process files belonging to episode only')

    def handle(self, force, emission, episode, verbosity, **kwargs):
        self.verbose = (verbosity > 1)

        for soundfile in SoundFile.objects.select_related().exclude(podcastable=False):
            if emission and soundfile.episode.emission.slug != emission:
                continue
            if episode and soundfile.episode.slug != episode:
                continue
            try:
                if soundfile.file is None or not os.path.exists(soundfile.file.path):
                    continue
            except ValueError:  # no file associated with it
                continue
            file_path = soundfile.get_format_path('waveform.json')
            if os.path.exists(file_path) and not force:
                continue
            self.create(soundfile)

    def create(self, soundfile):
        file_path = soundfile.get_format_path('waveform.json')
        # create a raw output, with a single channel and 4000Hz samples of 8bits
        cmd = ['sox', soundfile.get_format_path('ogg'), '-t', 'raw', '-c', '1', '-r', '4000',
                '-e' 'unsigned-integer', '-b', '8', '-']
        wave_array = subprocess.check_output(cmd)
        # reduce to 200 samples of max positive value
        wave_reduced = [max(x) for x in np.array_split(np.array([max(0, ord(x)-128) for x in wave_array]), 200)]
        json.dump(wave_reduced, file(file_path, 'w'))
